@@ -4,10 +4,27 @@ from shutil import rmtree
 
 
 class TreesappTester(unittest.TestCase):
+    def setUp(self) -> None:
+        from .testing_utils import get_test_data
+        # FASTA files
+        self.aa_test_fa = get_test_data("marker_test_suite.faa")
+        self.nt_test_fa = get_test_data("marker_test_suite.fna")
+
+        # Reference package pickles
+        self.refpkg_dir = get_test_data(os.path.join("refpkgs"))
+        self.mcra_pkl = get_test_data(os.path.join("refpkgs", "McrA_build.pkl"))
+        self.mcrb_pkl = get_test_data(os.path.join("refpkgs", "McrB_build.pkl"))
+        self.puha_pkl = get_test_data(os.path.join("refpkgs", "PuhA_build.pkl"))
+
+        # Output examples
+        self.ts_assign_output = get_test_data("test_output_TarA/")
+
+        return
+
     def tearDown(self) -> None:
         # Clean up output directories
         output_prefix = os.path.join(os.path.abspath("./"), "TreeSAPP_")
-        for test_name in ["assign", "train", "update", "evaluate", "create", "package", "purity", "MCC"]:
+        for test_name in ["assign", "train", "update", "evaluate", "create", "package", "purity", "MCC", "colour"]:
             output_dir = output_prefix + test_name
             if os.path.isdir(output_dir):
                 rmtree(output_dir)
@@ -17,10 +34,9 @@ class TreesappTester(unittest.TestCase):
         ref_pkgs = ["McrA", "M0702", "S0001"]
         from treesapp.commands import assign
         from treesapp.file_parsers import read_classification_table
-        from .testing_utils import get_test_data
-        assign_commands_list = ["--fastx_input", get_test_data("marker_test_suite.faa"),
+        assign_commands_list = ["--fastx_input", self.aa_test_fa,
                                 "--targets", ','.join(ref_pkgs),
-                                "--refpkg_dir", get_test_data(os.path.join("refpkgs")),
+                                "--refpkg_dir", self.refpkg_dir,
                                 "--num_procs", str(2),
                                 "-m", "prot",
                                 "--output", "./TreeSAPP_assign/",
@@ -28,15 +44,14 @@ class TreesappTester(unittest.TestCase):
                                 "--trim_align", "--overwrite", "--delete", "--svm"]
         assign(assign_commands_list)
         lines = read_classification_table("./TreeSAPP_assign/final_outputs/marker_contig_map.tsv")
-        self.assertEqual(16, len(lines))
+        self.assertEqual(15, len(lines))
         return
 
     def test_assign_dna(self):
         ref_pkgs = ["M0701", "M0702"]
         from treesapp.commands import assign
         from treesapp.file_parsers import read_classification_table
-        from .testing_utils import get_test_data
-        assign_commands_list = ["--fastx_input", get_test_data("marker_test_suite.fna"),
+        assign_commands_list = ["--fastx_input", self.nt_test_fa,
                                 "--targets", ','.join(ref_pkgs),
                                 "--num_procs", str(2),
                                 "-m", "dna",
@@ -47,6 +62,27 @@ class TreesappTester(unittest.TestCase):
         lines = read_classification_table("./TreeSAPP_assign/final_outputs/marker_contig_map.tsv")
         self.assertEqual(6, len(lines))
 
+    def test_colour(self):
+        from treesapp.commands import colour
+        colour_commands = ["-r", self.mcra_pkl, self.mcrb_pkl,
+                           "-l", "family",
+                           "-o", "./TreeSAPP_colour",
+                           "--filter", "Methanococcales",
+                           "--min_proportion", str(0.01),
+                           "--no_polyphyletic"]
+        colour(colour_commands)
+        self.assertEqual(True, True)
+
+    def test_colour_phenotypes(self):
+        from treesapp.commands import colour
+        from .testing_utils import get_test_data
+        colour_commands = ["-r", self.mcra_pkl,
+                           "-o", "./TreeSAPP_colour",
+                           "--palette", "viridis",
+                           "--taxa_map", get_test_data("Mcr_taxonomy-phenotype_map.tsv")]
+        colour(colour_commands)
+        self.assertTrue(True)
+
     def test_create(self):
         from treesapp.commands import create
         from treesapp.refpkg import ReferencePackage
@@ -74,35 +110,37 @@ class TreesappTester(unittest.TestCase):
 
     def test_create_eggnog(self):
         from treesapp.commands import create
-        from refpkg import ReferencePackage
+        from treesapp.refpkg import ReferencePackage
         from .testing_utils import get_test_data
         create_commands_list = ["--fastx_input", get_test_data("ENOG4111FIN.txt"),
-                                "--output", "./TreeSAPP_create_PuhA",
+                                "--output", "./TreeSAPP_create",
                                 "--refpkg_name", "PuhA",
                                 "--similarity", "0.90",
                                 "--profile", get_test_data("PuhA_search.hmm"),
                                 "--molecule", "prot",
                                 "--screen", "Bacteria,Archaea",
-                                "--num_proc", str(2),
+                                "--num_proc", str(4),
+                                "--raxml_model", "LG+F+R4",
                                 "--min_taxonomic_rank", 'p',
                                 "--stage", "support",
                                 "--trim_align", "--outdet_align", "--cluster", "--headless",
                                 "--overwrite", "--delete"]
         create(create_commands_list)
         test_refpkg = ReferencePackage()
-        test_refpkg.f__json = "./TreeSAPP_create_PuhA/final_outputs/PuhA_build.pkl"
+        test_refpkg.f__json = "./TreeSAPP_create/final_outputs/PuhA_build.pkl"
         test_refpkg.slurp()
         test_refpkg.validate()
         self.assertEqual(39, test_refpkg.num_seqs)
+        self.assertEqual("LG+F+R4", test_refpkg.sub_model)
         return
 
     def test_create_accession2lin(self):
         from treesapp.commands import create
-        from refpkg import ReferencePackage
+        from treesapp.refpkg import ReferencePackage
         from .testing_utils import get_test_data
         cmd_list = ["--fastx_input", get_test_data("PF01280_test.fasta"),
                     "--accession2lin", get_test_data("PF01280_test.tsv"),
-                    "--output", "./TreeSAPP_create_PF01280",
+                    "--output", "./TreeSAPP_create",
                     "--refpkg_name", "PF01280",
                     "--similarity", "0.95",
                     "--bootstraps", str(0),
@@ -114,7 +152,7 @@ class TreesappTester(unittest.TestCase):
                     "--trim_align", "--cluster", "--fast", "--headless", "--overwrite", "--delete"]
         create(cmd_list)
         test_refpkg = ReferencePackage()
-        test_refpkg.f__json = "./TreeSAPP_create_PF01280/final_outputs/PF01280_build.pkl"
+        test_refpkg.f__json = "./TreeSAPP_create/final_outputs/PF01280_build.pkl"
         test_refpkg.slurp()
         test_refpkg.validate()
         self.assertEqual(52, test_refpkg.num_seqs)
@@ -124,7 +162,7 @@ class TreesappTester(unittest.TestCase):
         from treesapp.commands import evaluate
         from .testing_utils import get_test_data
         evaluate_command_list = ["--fastx_input", get_test_data("McrA_eval.faa"),
-                                 "--refpkg_path", get_test_data(os.path.join("refpkgs", "McrA_build.pkl")),
+                                 "--refpkg_path", self.mcra_pkl,
                                  "--accession2lin", get_test_data("McrA_eval_accession_id_lineage_map.tsv"),
                                  "-o", "./TreeSAPP_evaluate",
                                  "-m", "prot",
@@ -137,12 +175,11 @@ class TreesappTester(unittest.TestCase):
 
     def test_abundance(self):
         from treesapp.commands import abundance
-        from file_parsers import read_classification_table
+        from treesapp.file_parsers import read_classification_table
         from .testing_utils import get_test_data
-        classification_table = os.path.join(get_test_data("test_output_TarA/"),
-                                            "final_outputs", "marker_contig_map.tsv")
+        classification_table = os.path.join(self.ts_assign_output, "final_outputs", "marker_contig_map.tsv")
         pre_lines = read_classification_table(get_test_data(classification_table))
-        abundance_command_list = ["--treesapp_output", get_test_data("test_output_TarA/"),
+        abundance_command_list = ["--treesapp_output", self.ts_assign_output,
                                   "--reads", get_test_data("test_TarA.1.fq"),
                                   "--reverse", get_test_data("test_TarA.2.fq"),
                                   "--pairing", "pe",
@@ -155,11 +192,11 @@ class TreesappTester(unittest.TestCase):
 
     def test_purity(self):
         from treesapp.commands import purity
-        from .testing_utils import get_treesapp_file, get_test_data
+        from .testing_utils import get_treesapp_file
         purity_command_list = ["--fastx_input", get_treesapp_file("dev_utils/TIGRFAM_seed_named.faa"),
                                "--extra_info", get_treesapp_file("dev_utils/TIGRFAM_info.tsv"),
                                "--output", "./TreeSAPP_purity",
-                               "--refpkg_path", get_test_data(os.path.join("refpkgs", "McrA_build.pkl")),
+                               "--refpkg_path", self.mcra_pkl,
                                "--trim_align", "--molecule", "prot", "-n", str(2)]
         purity(purity_command_list)
         self.assertEqual(True, True)
@@ -168,13 +205,13 @@ class TreesappTester(unittest.TestCase):
     def test_layer(self):
         from treesapp.commands import layer
         from .testing_utils import get_test_data
-        from file_parsers import read_classification_table
-        original_table = os.path.join(get_test_data("test_output_TarA/"), "final_outputs", "marker_contig_map.tsv")
-        layered_table = os.path.join(get_test_data("test_output_TarA/"), "final_outputs",
+        from treesapp.file_parsers import read_classification_table
+        original_table = os.path.join(self.ts_assign_output, "final_outputs", "marker_contig_map.tsv")
+        layered_table = os.path.join(self.ts_assign_output, "final_outputs",
                                      "extra_annotated_marker_contig_map.tsv")
         pre_lines = read_classification_table(get_test_data(original_table))
-        layer_command_list = ["--treesapp_output", get_test_data("test_output_TarA/")]
-        layer_command_list += ["--refpkg_dir", get_test_data("refpkgs/")]
+        layer_command_list = ["--treesapp_output", self.ts_assign_output]
+        layer_command_list += ["--refpkg_dir", self.refpkg_dir]
         layer(layer_command_list)
         post_lines = read_classification_table(get_test_data(layered_table))
         self.assertEqual(len(pre_lines), len(post_lines))
@@ -183,13 +220,13 @@ class TreesappTester(unittest.TestCase):
     def test_xmoa_layer(self):
         from treesapp.commands import layer
         from .testing_utils import get_test_data
-        from file_parsers import read_classification_table
+        from treesapp.file_parsers import read_classification_table
         classification_table = os.path.join(get_test_data("p_amoA_FunGene9.5_isolates_assign/"),
                                             "final_outputs", "marker_contig_map.tsv")
         pre_lines = read_classification_table(get_test_data(classification_table))
         layer_command_list = ["--colours_style", get_test_data("XmoA_Function.txt"),
                               "--treesapp_output", get_test_data("p_amoA_FunGene9.5_isolates_assign/"),
-                              "--refpkg_dir", get_test_data("refpkgs/")]
+                              "--refpkg_dir", self.refpkg_dir]
         layer(layer_command_list)
         post_lines = read_classification_table(get_test_data(classification_table))
         self.assertEqual(len(pre_lines), len(post_lines))
@@ -197,15 +234,14 @@ class TreesappTester(unittest.TestCase):
 
     def test_update_resolve(self):
         from treesapp.commands import update
-        from refpkg import ReferencePackage
+        from treesapp.refpkg import ReferencePackage
         from .testing_utils import get_test_data
         update_command_list = ["--fastx_input", get_test_data("Photosynthesis/PuhA/ENOG4111FIN_PF03967_seed.faa"),
-                               "--refpkg_path", get_test_data(os.path.join("refpkgs", "PuhA_build.pkl")),
+                               "--refpkg_path", self.puha_pkl,
                                "--treesapp_output", get_test_data("assign_SwissProt_PuhA/"),
                                "--output", "./TreeSAPP_update",
                                "--num_proc", str(4),
                                "--molecule", "prot",
-                               "-b", str(0),
                                "--trim_align", "--cluster", "--fast", "--headless",
                                "--overwrite", "--delete", "--skip_assign", "--resolve"]
         update(update_command_list)
@@ -218,10 +254,10 @@ class TreesappTester(unittest.TestCase):
 
     def test_update_seqs2lineage(self):
         from treesapp.commands import update
-        from refpkg import ReferencePackage
+        from treesapp.refpkg import ReferencePackage
         from .testing_utils import get_test_data
         update_command_list = ["--fastx_input", get_test_data("Photosynthesis/PuhA/ENOG4111FIN_PF03967_seed.faa"),
-                               "--refpkg_path", get_test_data(os.path.join("refpkgs", "PuhA_build.pkl")),
+                               "--refpkg_path", self.puha_pkl,
                                "--treesapp_output", get_test_data("assign_SwissProt_PuhA/"),
                                "--seqs2lineage", get_test_data("SwissProt_PuhA_seqs2lineage.txt"),
                                "--output", "./TreeSAPP_update",
@@ -246,7 +282,7 @@ class TreesappTester(unittest.TestCase):
         train_command_list = ["--fastx_input", get_test_data("ENOG4111FIN.txt"),
                               "--annot_map", get_test_data("ENOG4111FIN_annot_map.tsv"),
                               "--output", output_dir_path,
-                              "--refpkg_path", get_test_data(os.path.join("refpkgs", "PuhA_build.pkl")),
+                              "--refpkg_path", self.puha_pkl,
                               "--accession2lin", get_test_data("ENOG4111FIN_accession_id_lineage_map.tsv"),
                               "--num_proc", str(4),
                               "--molecule", "prot",
@@ -267,10 +303,9 @@ class TreesappTester(unittest.TestCase):
 
     def test_package(self):
         from treesapp.commands import package
-        from .testing_utils import get_test_data
         command_list = ["view",
                         "lineage_ids",
-                        "--refpkg_path", get_test_data(os.path.join("refpkgs", "McrA_build.pkl")),
+                        "--refpkg_path", self.mcra_pkl,
                         "--output", "./TreeSAPP_package"]
         package(command_list)
         self.assertEqual(True, True)
@@ -282,6 +317,7 @@ class TreesappTester(unittest.TestCase):
         cmd = ["--fastx_input", get_test_data("EggNOG_McrA.faa"),
                "--annot_map", get_test_data("EggNOG_McrA_annot_map.tsv"),
                "--output", "./TreeSAPP_MCC",
+               "--refpkg_dir", self.refpkg_dir,
                "--targets", "McrA",
                "--molecule", "prot",
                "--tool", "treesapp",
